import matplotlib.pyplot as plt
import matplotlib.colors as mcolor
import numpy as np


def disp(img, norm=None, pct=.5):
    """Display an image"""

    vmin, vmax = np.percentile(img.flatten(), [pct, 100-pct])
    norm = norm or mcolor.Normalize(vmin, vmax)

    plt.imshow(
        img,
        cmap=plt.cm.YlGnBu_r,
        origin="lower",
        interpolation="nearest",
        norm = norm,
    )





# -*- coding: utf-8 -*-
"""
Created on Mon Nov  2 07:24:17 2020

@author: fergal
"""

from ipdb import set_trace as idebug
from pdb import set_trace as debug
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

from . import const

import scipy.integrate as spInt
#import synphot.units as su
import astropy.units as u


def synthPhot(model_wavel, model_flux, filter_wavel, filter_transmission):
    """Perform synthetic photometry on a model

    Inputs
    ---------
    model_wavel
        (1d np array) Wavelenth of model. Any units
    model_flux
        (1d np array) Flux at each wavelength, any units
    filter_wavel
        (1d np array) Wavelenghts of filter. Must be same units
        as model_wavel, but need not be evaulated at the same values
    filter_transmission
        (1d np array). Fraction of light transmitted by the filter
        at this wavelength. Values should be in the range [0,1]

    The wavelength range covered by the model must bracket the
    wavelength range covered by the filter. You may need to
    trim the ends of your filter (if they are all zeros) to
    make this pre-condition true.

    This function works best if the filter is sampled at least
    as frequently as the model. If this is not true, you should
    interpolate the filter to the model wavelength grid before
    passing it to this function.
    """
    #Check object spectrum brackets the filter
    assert np.all(np.diff(filter_wavel) > 0)
    assert np.all(np.diff(model_wavel) > 0)

    #Check the model spans the range of the filter
    assert model_wavel[0] < filter_wavel[0]
    assert filter_wavel[-1] < model_wavel[-1]

    assert np.min(filter_transmission) >= 0
    assert np.max(filter_transmission) <= 1

    model_flux = np.interp(filter_wavel, model_wavel, model_flux)
    transmitted_flux = model_flux * filter_transmission

    flux = spInt.trapz(transmitted_flux, filter_wavel)
    flux /= spInt.trapz(filter_transmission, filter_wavel)
    return flux



def convertKoesterToJy(data, radius_m, dist_parsec):
    """Convert Koester's file so the wavelengths are in metres
    and the fluxes in Janskys
    """

    wavelength_ang = data[:,0]
    flux = data[:,1]

    wavel_m = wavelength_ang * 1e-10

    # Koester's flux  (4x Eddington) = 4*PI*Intensity
    # Intensity is flux per solid angle. Flux is Intensity
    # integrated over half a solid angle = PI*Intensity
    flux *= 4

    # Convert from flux in erg/cm^2/s/cm to W/m^2/m
    flux *= 1e-1

    # Convert parsecs to metres
    dist_m = dist_parsec * const.parsec

    flux *= (radius_m / dist_m) ** 2

    flux = convertFlambdaToFnu(wavel_m, flux)
    flux /= const.jansky

    data[:,0] = wavel_m
    data[:,1] = flux
    return data


def convertNextGenToJy(st, radius_m, dist_parsec):
    """Convert NextGen models by Allard to Jansky

    These models are described in Hautschild et al 1999
    """
    w = st[:,0]
    f = st[:,1]

    w = w * u.angstrom
    f = f * u.erg  / u.second / u.cm**2 / u.cm

    fjy = su.convert_flux(w, f, u.jansky)
    radius = .2  * u.solRad
    dist = 10 * u.parsec

    fjy *= (radius/dist)**2

    out = np.empty_like(st)
    out[:,0] = w.to(u.meter)
    out[:,1] = fjy.to(u.jansky)
    return out



def convertKuruczToJy(st):
    """
    Convert Kurucz models to Jy.

    The best ref for these I've found is

    https://archive.stsci.edu/hlsps/reference-atlases/cdbs/grid/k93models/AA_README

    The NextGen models have finer spectral resolution
    """
    w = st[:,0]
    f = st[:,1]

    w = w * u.angstrom
    f = f * su.FLAM

    fjy = su.convert_flux(w, f, u.jansky)
    radius = .2  * u.solRad
    dist = 10 * u.parsec

    fjy *= (radius/dist)**2

    out = np.empty_like(st)
    out[:,0] = w.to(u.meter)
    out[:,1] = fjy.to(u.jansky)
    return out


def convertFlambdaToFnu(wavel_m, flux_lam):

    factor = (wavel_m **2) / const.speedOfLight
    flux_nu = flux_lam * factor
    return flux_nu



def sumModels(w1, f1, w2, f2):
    """Some two spectra, accounting for differences in wavelenth reticules.
    
    This is quick an dirty, you may want something more sophisticated
    """
    assert len(w1) == len(f1)
    assert len(w2) == len(f2)
    f3 = np.interp(w1, w2, f2)
    out = np.vstack([w1, f1+f3]).transpose()
    return out







import numpy as np 

"""
Initial-final mass relations for white dwarf stars according to various authors

Input the final, white dwar, mass, and get an estimate of the progenitor, main-sequence
mass. 

These references are all 15 years old at this point, and could do with some 
updating, but for approximate work they should be just fine.



"""


def estimate_main_sequence_lifetime_Gyr(mass_solar, mass_unc=None):
    """
    Based on Eqn 1.88 of "Stellar Interiors: Physical Principles, Structure, and 
    Evolution"  by Hansen & Kawaler 
    """
    age =  10 * mass_solar**(-2.5)
    
    if mass_unc is None:
        return age

    age_unc = mass_unc * 10 * (2.5) * mass_solar ** (-3.5) 
    return age, age_unc
    

def williams07_ifmr(wd_mass_solar, mass_unc=None):
    """
    Taken from Williams & Bolte 2007AJ....133.1490W 
    """
    
    a, da = .132, .017     #//slope = a+bx
    b, db =.33, .07
    
    return linear_ifmr(wd_mass_solar, mass_unc, a, da, b, db)



def linear_ifmr(Mf_solar, dMf, a0, da0, b0, db0):
    """Compute inital progenitor mass from final white dwarf mass using a linear
    initial-final mass ratio
    
    Inputs
    -----------
    Mf_solar, dMf
        (float) Final, WD, mass in solar masses, with uncertainty
    """
    
    Minit_solar = (Mf_solar - a0) / b0
    if dMf is None:
        return Minit_solar 
    
    
    val1 = dMf**2 + da0**2;
    val1 /= b0**2;

    val2 = (Mf_solar - a0)**2 * db0**2;
    val2 /= b0**4;

    Minit_unc = np.sqrt(val1 + val2);
    return Minit_solar, Minit_unc
        




#//Because so many IFMR's are linear, I've abstracted out a function to
#//do this simplest calculation.
#void linearIFMR(double Mf, double dMf, double a0, double da0, double a1, double da1, double *Mi, double *dMi)
#{
    #double val1, val2;
    
    #(*Mi) = (Mf-a0)/a1;
    
    #//If user doesn't request uncertainty, don't calculate it
    #if(dMi != NULL)
    #{
        #val1 = sqr(dMf) + sqr(da0);
        #val1 /= sqr(a1);

        #val2 = sqr(Mf-a0) * sqr(da1);
        #val2 /= pow(a1,4);

        #(*dMi) = sqrt(val1 + val2);
    #}
#}


#void williams07IFMR(double wdmass, double dwdmass, double *starmass, double *dstarmass)
#{
    #double val, dval;
    #double a=.132, da=.017;     //slope = a+bx
    #double b=.33, db=.07;

    #(*starmass) = (wdmass - b)/a;

    #val = (wdmass-b)/sqr(a)*da;
    #dval = sqr(val);

    #val = db/a;
    #dval += sqr(val);

    #val = dwdmass/a;
    #dval += sqr(val);

    #(*dstarmass) = sqrt(dval);
#}



#//Ferrario05 has two fits to the IMFR, I only use the linear case
#// 2005MNRAS.361.1131F 
#void ferrario05IFMR(double wdmass, double dwdmass, double *starmass, double *dstarmass)
#{
    #double val, dval;
    #double a=.10038, da=.00518;     //slope = a+bx
    #double b=.43443, db=.01467;

    #(*starmass) = (wdmass - b)/a;

    #val = (wdmass-b)/sqr(a)*da;
    #dval = sqr(val);

    #val = db/a;
    #dval += sqr(val);

    #val = dwdmass/a;
    #dval += sqr(val);

    #(*dstarmass) = sqrt(dval);
#}



#void kalirai07IFMR(double wdmass, double dwdmass, double *starmass, double *dstarmass)
#{
    #double a0, da0;
    #double a1, da1;
    
    #//y = a0 + a1*x
    #a0=0.394;
    #da0=0.025;
    
    #a1= 0.109;
    #da1= 0.007;
    
    #linearIFMR(wdmass, dwdmass, a0, da0, a1, da1, starmass, dstarmass);
#}



#void dobbie06IFMR(double wdmass, double dwdmass, double *starmass, double *dstarmass)
#{
    #double a0, da0;
    #double a1, da1;
    
    #//y = a0 + a1*x
    #a0=0.289;
    #da0=0.051;
    
    #a1= 0.133;
    #da1= 0.015;
    
    #linearIFMR(wdmass, dwdmass, a0, da0, a1, da1, starmass, dstarmass);
#}



from ipdb import set_trace as idebug

from glob import glob 
import numpy as np
import os 

def lmap(function, *array):
    """Perform a map, and package the result into a list, like what map did in Py2.7"""
    return list(map(function, *array))


def loadFilters(pattern=None):
    """
    Load filter tracings

    The filter tracings are stored on disk. This function
    loads them all to a dictionary. The keys are

    Input
    ------
    pattern
        (str) A string that glob can use to match a set of files, e.g
        filters/*.dat. If not given, the function tries to guess the correct
        value.
        
    Returns
    ---------
    A dict of 2d numpy arrays. The columns of the arrays are
        * wavelength (in metres)
        * transparency (0=> opaque, 1=> transparent)
        
    The keys are : {system}_{band}, e.g sloan_u or miri_f1000w. The 
    values are 2d numpy arrays with columns
    """
    if pattern is None:
        pattern = os.path.split(__file__)[0]
        pattern = os.path.join(pattern, 'filter_tracings/*.dat')

    filterFiles = glob(pattern)

    if not filterFiles:
        raise ValueError(f"No filter tracings found in {pattern}. Please specify glob pattern as input arg")
    filterNames = lmap(getFilterName, filterFiles)


    filters = dict()
    for fn in sorted(filterFiles):
        fName = getFilterName(fn)
        data = np.loadtxt(fn)
        filters[fName] = data
    return filters


def getFilterName(fn):
    fName = os.path.split(fn)[-1]
    fName = os.path.splitext(fName)[0]
    return fName


def getZeroPointFluxes_Jy():
    """
    Optimal and NIR taken from 
    http://astroweb.case.edu/ssm/ASTR620/mags.html
    The JHK zeros are from the original paper and may not 
    match 2mass exactly.
    
    
    SDSS is on the AB scale, so the 0 mag in each
    band should be the same value. SDSS u and z are slightly off
    from AB, by amounts listed in the comments. If you
    care about accuracy at the <0.05 level you should
    correct those zero points.
    See https://www.sdss.org/dr12/algorithms/fluxcal/
        
    """
    zero = {
        'johnson-U': 1810,
        'johnson-B': 4260,
        'johnson-V': 3640,
        'johnson-R': 3080,
        'johnson-I': 2550,
        'twomass-J': 1600,
        'twomass-H': 1080,
        'twomass-Ks': 670,
        'sloan-u': 3631,  # u_sdss = u_ab + 0.04
        'sloan-g': 3631,
        'sloan-r': 3631,
        'sloan-i': 3631,
        'sloan-z': 3631, # z_sdss = z_ab - 0.02
    }
    
    zero.update(jwstZeroPointFluxes_Jy())
    return zero


def jwstZeroPointFluxes_Jy():
    """Returns a dictionary of flux in Janskies of Vega in some JWST bands

    Taken from
    http://svo2.cab.inta-csic.es/svo/theory/fps3/index.php?mode=browse&gname=JWST&gname2=NIRCam&asttype=

    """
    zero = {
                #NIRCam
                'nircam-F070W' : 2748.39,
                'nircam-F090W' : 2249.58,
                'nircam-F115W' : 1741.77,
                'nircam-F140M' : 1309.05,
                'nircam-F150W' : 1175.1,
                'nircam-F162M' : 1041.79,
                'nircam-F164N' : 1007.79,
                'nircam-F150W2' : 933.57,
                'nircam-F182M' :  858.76,
                'nircam-F187N' : 813.41,
                'nircam-F200W' : 759.59,
                'nircam-F210M' : 701.37,
                'nircam-F212N' : 690.9,
                'nircam-F250M' : 515.84,
                'nircam-F277W' : 430.19,
                'nircam-F300M' : 377.25,
                'nircam-F323N' : 329.21,
                'nircam-F322W2' : 318.55,
                'nircam-F335M' :  305.6,
                'nircam-F356W' : 272.62,
                'nircam-F360M' : 266.13,
                'nircam-F405N' : 212.51,
                'nircam-F410M' : 213.27,
                'nircam-F430M' : 195.51,
                'nircam-F444W' : 184.42,
                'nircam-F460M' : 168.3,
                'nircam-F466N' : 162.02,
                'nircam-F470N' : 163.77,
                'nircam-F480M' : 157.04,

                #MIRI
                'miri-F560W'  : 116.38,
                'miri-F770W'  :  64.83,
                'miri-F1000W' :  38.99,
                'miri-F1130W' :  30.42,
                'miri-F1280W' :  23.79,
                'miri-F1500W' :  17.28,
                'miri-F1800W' :  12.18,
                'miri-F2100W' :   9.11,
                'miri-F2550W' :   6.14,
    }
    return zero


def convertMagToJy(mag, zeroPointFlux):
    """Convert a magnitude to a Jansky

    Inputs
    ------
    mag
        (float or np.array) Value(s) to convery
    zeroPointFlux
        (float) Flux in Janskies corresponding to mag=0
        in that system
    """

    return zeroPointFlux * 10**(-mag/2.5)  


def convertJyToMag(flux, zeroPointFlux):
    return -2.5 * np.log10(flux/zeroPointFlux)

def jwstBackgroundLimitsLow_Jy() -> dict:
    """Background limits for JWST for regions of the sky with low background
    
    MIRI Limits taken from Glasse et al (2015) 127, 675, Table 3
    https://iopscience.iop.org/article/10.1086/682259
    """

    limits = {
        'miri-F560W'  : 0.16e-6,
        'miri-F770W'  : 0.25e-6,
        'miri-F1000W' : 0.54e-6,
        'miri-F1130W' : 1.35e-6,
        'miri-F1280W' : 0.84e-6,
        'miri-F1500W' : 1.39e-6,
        'miri-F1800W' : 3.46e-6,
        'miri-F2100W' : 7.09e-6,
        'miri-F2550W' : 26.2e-6,
    }
    return limits


def jwstBackgroundLimitsHigh_Jy() -> dict:
    """Background limits for JWST for regions of the sky with low background
    
    MIRI Limits taken from Glasse et al (2015) 127, 675, Table 3
    https://iopscience.iop.org/article/10.1086/682259
    """

    limits = {
        'miri-F560W'  :  0.16e-6,
        'miri-F770W'  :  0.25e-6,
        'miri-F1000W' :  0.56e-6,
        'miri-F1130W' :  1.43e-6,
        'miri-F1280W' :  1.00e-6,
        'miri-F1500W' :  1.94e-6,
        'miri-F1800W' :  4.87e-6,
        'miri-F2100W' :  9.15e-6,
        'miri-F2550W' : 30.80e-6,
    }
    return limits



# -*- coding: utf-8 -*-
import re
import math
import numpy as np
import datetime
import inspect


def jrdToBkjd(val):
    return val+67.0

def bkjdToMbjd(val):
    return val+54832.5

def mbjdToBkjd(val):
    return val-54832.5

def rbjdToBkjd(val):
    return toFloat(val)-54833.0


def bjdToBkjd(val):
    return val - 2454833.0

def fixKoiString(val):
    if re.match("^K\d\d\d\d\d.\d\d", val):
        #Already in the right format
        return val

    num = toFloat(val)
    if num==0:
        raise ValueError("Can't parse %s into a KOI" %(str(val)))

    out = "K%08.02f" %(num)
    return re.sub("\.00", ".01", out)


def minorKoi(koiString):
    reObj = re.match("K.....\.(\d\d)", koiString)
    return int(reObj.group(1))

def majorKoi(koiString):
    reObj = re.match("K(.....)\.\d\d", koiString)
    return int(reObj.group(1))


def createTceId(kepid, planetNum):
    """Create a unique number (for a quarter) for a TCE based on its kepid and
    planet number in DV. Useful for comparing lists of TCEs

    Input:
    kepid       (float or 1d array)
    planetNum   (float or 1d array)

    Return
    float or 1d array depending on input
    """

    tceId = 100*(kepid + planetNum/100.)

    #If inputs are numpy, we can cast to an int
    if isinstance(tceId, np.ndarray):
        tceId = np.round(tceId).astype(np.int64)
    return tceId



def toFloat(strr):
    try:
        return float(strr)
    except ValueError:
        return 0
    except TypeError:
        return 0


def isScalar(x):
    """Returns true is x is a single object, but false if it
    is an array.

    If isScalar(x), then len(x) throws an exception
    """

    return not hasattr(x, "__len__")


def ppmToMilliMag(ppm):
    assert(ppm < 1e6)

    val = 1- ppm/1e6
    return -2.5*math.log10(val)

def milliMagToPpm(mmag):
    val = 1 - 10**(mmag/-2500)
    return 1e6*val


def isClose(a,b, tol):
    """Synonym for checkClose"""
    return checkClose(a,b, tol)

def checkClose(a, b, tol=1e-5):
    a = toFloat(a)
    b = toFloat(b)
    if math.fabs(a-b) > tol:
        return False
    return True


def getch():
    raw_input()


def grep(pattern, filename):
    """Simple replacement for a command line grep.

    Sometimes you want to check for the existence of something in a
    file without going through the hassle of reading it in. This
    is a simple replacement for the grep tool. It doesn't have the
    power of it's command line brother, but it's often enough.
    """

    out = []

    fp = open(filename)
    text = fp.readlines()
    fp.close()

    expr = re.compile(pattern)
    for line in text:
        if re.search(expr, line):
            out.append(line)
    return out



def printse(val, err, sigdig=2, pretty=True):
    """Return a value and its uncertainty in SI format, and
    scientific notation, eg 12.345(67)e7

    Input
    val, err (float) The value and it's associated uncertainty
    sigdig (int)     Number of significant digits to print
    pretty (bool)    If true print the number as 12.345(67).
                     If false, return 12.345 0.0067
                     pretty option not implemented yet

    Returns:
    A string
    """

    vex=orderOfMag(float(val))
    eex=orderOfMag(float(err))

    val /= 10**vex
    err /= 10**eex

    big = max(vex, eex)
    val /= 10**(big-vex)
    err /= 10**(big-eex)

    strr = printe(val, err, sigdig=sigdig, pretty=pretty)
    if pretty:
        strr = "%se%02i" %(strr, big)
    else:
        tmp = strr.split()
        strr= "%se%02i %se%02i" %(tmp[0], big, tmp[1], big)

    return strr


def printe(val, err, sigdig=2, pretty=True):
    """Return a value and its uncertainty in SI format, 12.345(67)

    Input
    val, err (float) The value and it's associated uncertainty
    sigdig (int)     Number of significant digits to print
    pretty (bool)    If true print the number as 12.345(67).
                     If false, return 12.345 0.0067

    Returns:
    A string
    """

    val = float(val)
    err = float(err)

    if err == 0:
        strr = "%.6f 0" %( val)
        return strr

    #Significant digits in the value
    sd = int(max(0, sigdig-1-orderOfMag(err)))


    if pretty:
        if err > 1 and err < 10:
            strr = " %.*f(%.1f)" %(sd, val, err)
        else:
            if sd !=0:
                err /= pow(10, orderOfMag(err)-1)
            #err /= pow(10, orderOfMag(err)-2)
            strr = "%.*f(%g)" %(sd,val,round(err))
    else:
        if err > 1 and err < 10:
            strr = " %.*f %.1f" %(sd, val, err)
        else:
            #err /= pow(10, orderOfMag(err)-1)
            #strr = " %.*f(%g)", sd, val, math.rint(err)
            strr = "%.*f %.*f" %(sd,val,sd,err)

    return strr


def orderOfMag(val):
    """Return the order of magnitude of value

    e.g orderOfMag(9999) is 3

    Input
    val (float)

    Return
    (int)
    """

    return int(math.floor(math.log10(math.fabs(float(val)))))


def respace(stringList, sep=None):
    """Take a list of strings and add white space so the columns
    are left justified.
    """

    #Measure the maximum width of each column
    size = dict()
    for line in stringList:
        if re.search("^#", line):
            continue    #Skip comments

        words = line.split(sep)

        for i, w in enumerate(words):
            if i in size:
                size[i] = max(size[i], len(w))
            else:
                size[i] = len(w)

    #Loop through a second time, format-printing each element
    outList = []
    for line in stringList:
        if re.search("^#", line):
            outList.append(line)  #Don't reformat comments
        else:
            outLine = []
            for i, w in enumerate(line.split(sep)):
                newWord= "%-*s" %(size[i], w)
                outLine.append(newWord)
            outList.append(" ".join(outLine))

    return outList



def decToSeg(ra, dec, sep=" "):
    """Convert ra and dec to a segasdecimal string

    Input
    ra float
    dec float

    Returns
    A string
    """

    #Just in case, force ra and dec to be floats
    ra = float(ra)
    dec= float(dec)

    #First the ra
    des = ra*86400/360. #convert to arcsec
    hr = math.floor(des /3600.)
    mn = math.floor( (des - hr*3600)/60)
    sc = des - hr*3600 -mn*60
    ra = "% 3.0f%s%02.0f%s%05.2f" %(hr, sep, mn, sep,sc)

    #Now the declination
    if dec>0:
        deg = math.floor(dec)
        sign="+"
    else:
       deg = math.ceil(dec)
       sign = "-"

    minsec = abs(dec - deg)
    deg = abs(deg)

    mn = math.floor(minsec*60)
    sc = 3600*(minsec - mn/60)
    dec = "%s%02.0f%s%02.0f%s%04.1f" %(sign, deg,sep, mn,sep, sc)

    return "%s %s" %(ra,dec)


def segToDec(seg,  sep="\s+"):
    """Convert a string of 6 numbers to a decimal ra and dec

    Input
    seg A string of the form hh mm ss.ss +dd mm ss.
    A string with 6 numbers separated by whitespace will do, this function
    does it's best to handle it.

    Returns
    A tuple of (ra, dec)
    """

    a1,a2,a3,a4,a5,a6 = re.split(sep, seg, maxsplit=6)

    #Calculate ra in decimal hours and then convert
    ra = float(a1)
    ra += float(a2)/60.
    ra += float(a3)/3600
    ra *= 360/24.

    a4 = float(a4)
    if a4 < 0:
        dec = -1*a4
        dec += float(a5)/60.
        dec += float(a6)/3600
        dec *= -1
    else:
        dec = a4
        dec += float(a5)/60.
        dec += float(a6)/3600

    return (ra, dec)




def testInRange(val, lwr=None, upr=None, msg=""):
    """Tests that val in in the range [lwr, upr].

    Returns an empty string if true, or a string if false.
    If either lwr or upr are None, this test is not performed.
    """

    fail=0
    if lwr is not None:
        if val < lwr:
            fail = 1

    if upr is not None:
        if val > upr:
            fail = 1

    if fail == 1:
        return "%s: %g not in range [%g, %g] " %(msg, val, lwr, upr)
    return ""



def findOverlap( left, right, output="All"):
    """Find the elements that exist in both sets, or one but not the other

    Inputs
    left
    right   Any iterable object e.g a list or tuple
    output  (string) All|Left|Right|Both
            Left: return a set of all objects in left, but not in right (A')
            Right: return a set of all objects in right, but not in left (B')
            Both: return a set of all objects in both left and right (A ^ B)
            All: Return a tuple of (Left, Both, Right)

    Return
    A set object, or a tuple, depending on the value of output
    """

    A = set(left)
    B = set(right)

    if output == "Left":
        return A - B
    elif output == "Right":
        return B - A
    elif output == "Both":
        return A & B
    else:
        return (A-B, A & B, B-A)


def sortedIndices(self, data):
    """Neat way to do np.argsort() on non numpy lists
    Taken from http://code.activestate.com/recipes/306862/
    """
    return sorted(range(len(data)), key = data.__getitem__)


def createHeader(headerStr, textList=None, columnNames=None, **kwargs):
    """Create a useful header for a text file

    Inputs:
    ------------
    headerStr
        (str) A one line summary of the file contents
    textList
        (list) A list of strings providing additional documentation
    columnNames
        (list) A list of strings giving meta data on the columns

    Options Inputs:
    ----------------
    Any optional arguments are written to the header as key:value
    For example:
    printHeader(... temp="26", units="celcius") produces a header that
    includes the lines
    temp: 26
    units: celcius

    To pass a dictionary of parameters do
    params=dict()
    ...
    printHeader(... **params)

    Returns:
    -------------
    An array of strings.

    Notes:
    -------------
    Function searches for __versio  n__ and __URL global variables in
    your file and prints those to the header if available.
    """
    frame = inspect.currentframe().f_back
    if frame.f_globals['__name__'] == 'kepler.tools':
        frame = frame.f_back

    try:
        idStr = frame.f_globals['__version__']
    except KeyError:
        idStr = frame.f_globals['__file__']

    try:
        urlStr = frame.f_globals['__URL__']
    except KeyError:
        urlStr = "<__URL__ not set>"

    try:
        frameInfo = inspect.getframeinfo(frame)
        funcName = frameInfo[2]
        lineno = frameInfo[1]
    except KeyError:
        funcName = "<Unknown>"
        lineno = "0"

    out = []
    out.append("#%s" %(headerStr))
    out.append("#Created by: %s" %(idStr))
    out.append("#%s" %(urlStr))
    out.append("#Function name: %s" %(funcName))
    out.append("#Line Num: %s" %(lineno))
    out.append("#At: %s" %(datetime.datetime.now()))

    if textList is not None:
        for text in textList:
            out.append("#%s" %(text))

    if columnNames is not None:
        out.append("#Column definitions:")
        for i, line in enumerate(columnNames):
            out.append("#Col %i: %s" %(i, line))

    if len(kwargs.keys()) > 0:
        out.append("#Parameters used")
    for k in kwargs.keys():
        out.append("#%s: %s" %(k, kwargs[k]))
    out.append("#")

    if columnNames is not None:
        words = map( lambda x: x.split()[0], columnNames)
        out.append("#" + " ".join(words))

    return out


def printHeader(fp, headerStr, textList=None, **kwargs):
    """Print a useful header to file pointed to by fp

    Inputs:
    fp          (String or file). If str, function attempts to open
                and write to a file of that name. If file objects,
                then must be writable
    headerStr   (str) A one line summary of the file contents
    textList    (list) A list of strings providing additional documentation

    Options Inputs:
    Any optional arguments are written to the header as key:value
    For example:
    printHeader(... temp="26", units="celcius") produces a header that
    includes the lines
    temp: 26
    units: celcius

    To pass a dictionary of parameters do
    params=dict()
    ...
    printHeader(... **params)

    Returns:
    File pointer. Either fp, or the file pointer to the file called
    fp depending on the type of the input

    Output:
    Text is written to the file pointed to by fp, and fp is kept open.

    Notes:
    Function searches for __versio  n__ and __URL global variables in
    your file and prints those to the header if available.
    """

    if not hasattr(fp, 'write'):
        fp = open(fp, 'w')

    fp.write("\n".join( createHeader(headerStr, textList, **kwargs)))
    fp.write("\n")

    return fp

def compareDicts(dict1, dict2, tupleArray):
    """Compare the values in dict1 to dict2.

    Inputs:
    dict1, dict2:   (Dicts) Dictionaries to be compared
    tupleList       (List) A list of tuples

    Return:
    An empty string if dicts agree. An diagnostic string if they don't.

    Notes:
    Each tuple consists of (key1, key2, function, arg)
    The value of dict1[key1] is compared to dict2[key2].

    If function=None, the keys are compared with a string comparison
    Otherwise, function is called with the arguments
    (dict1[key1], dict2[key2], arg)

    For example, to compare the values as floating point numbers with
    a tolerance of 1e-6 use the tuple (key1, key2, checkClose, 1e-6)

    The comparison function should return True if the values are
    equal in the desired sense, and False if they disagreee
    """

    msg=[]
    for (k1, k2, ptest, arg) in tupleArray:
        if ptest is None:
            if str(dict1[k1]) != str(dict2[k2]):
                msg.append("strComp: %s( %s ) != %s( %s ): " %(k1, dict1[k1], k2, dict2[k2]))
        else:
            err = ptest(dict1[k1], dict2[k2], arg)
            if  err != "":
                testName = (str(ptest).split())[1]
                msg.append("%s: %s( %s ) != %s( %s ) [%s] " %(testName, k1, dict1[k1], k2, dict2[k2], err))

    return msg



def hypotUnc(a, da, b, db):
    """Compute the hypotenuse and its uncertainty"""

    y = np.hypot(a,b)

    c1 = a*da
    c2 = b*db
    dy = np.hypot(c1, c2)/ y

    return y, dy


def haversine(x):
    """Return the haversine of an angle

    haversine(x) = sin(x/2)**2, where x is an angle in radians
    """
    y = .5*x
    y = math.sin(y)
    return y*y


def sphericalAngSep(ra0, dec0, ra1, dec1, radians=False):
    """
        Compute the spherical angular separation between two
        points on the sky.

        //Taken from http://www.movable-type.co.uk/scripts/gis-faq-5.1.html

        NB: For small distances you can probably use
        sqrt( dDec**2 + cos^2(dec)*dRa)
        where dDec = dec1 - dec0 and
               dRa = ra1 - ra0
               and dec1 \approx dec \approx dec0
    """

    if radians==False:
        ra0  = math.radians(ra0)
        dec0 = math.radians(dec0)
        ra1  = math.radians(ra1)
        dec1 = math.radians(dec1)

    deltaRa= ra1-ra0
    deltaDec= dec1-dec0

    val = haversine(deltaDec)
    val += math.cos(dec0) * math.cos(dec1) * haversine(deltaRa)
    val = min(1, math.sqrt(val)) ; #Guard against round off error?
    val = 2*math.asin(val)

    #Convert back to degrees if necessary
    if radians==False:
        val = math.degrees(val)

    return val


def sphericalAngBearing(ra0, dec0, ra1, dec1, radians=False):
    sin = math.sin
    cos = math.cos
    atan  = math.atan2

    if radians==False:
        ra0  = math.radians(ra0)
        dec0 = math.radians(dec0)
        ra1  = math.radians(ra1)
        dec1 = math.radians(dec1)

    dLong = ra1 - ra0
    a = sin(dLong)*cos(dec1)
    b = cos(dec1)*sin(dec1) - sin(dec0)*cos(dec1)*cos(dLong)
    bearing = atan(a, b)

    if radians==False:
        bearing = math.degrees(bearing)

    return bearing


def sphericalAngDestination(ra0_deg, dec0_deg, bearing_deg, dist_deg):
    sin = math.sin
    cos = math.cos
    asin  = math.asin
    atan2 = math.atan2

    phi1 = math.radians(dec0_deg)    #Latitude
    lambda1 = math.radians(ra0_deg)    #Longitude
    d = math.radians(dist_deg)      #Distance in radians
    theta = math.radians(bearing_deg)

    phi2 = sin(phi1)*cos(d)
    phi2 += cos(phi1)*sin(d)*cos(theta)
    phi2 = asin(phi2)

    a = sin(theta)*sin(d)*cos(phi1)
    b = cos(d) - sin(phi1)*sin(phi2)
    #print "A=%.7f b=%.7f" %(a,b)
    #import pdb; pdb.set_trace()
    lambda2 = lambda1 + atan2(a,b)

    ra2_deg = math.degrees(lambda2)
    dec2_deg = math.degrees(phi2)
    return ra2_deg, dec2_deg





import numpy as np


def downsample_2darray(arr, sub_pixels_per_row, sub_pixels_per_col):
    """Downsample a 2d array to a smaller array.

    Suppose you have an nxn array, and you want to bin it 2x2 to 
    make a smaller array. You produce the smaller array by::

        >>> new_array = downsample_2darray(arr, 2, 2)

    This is useful when you have an oversampled image that you want
    to resample to, eg, pull out faint features.
    
    Inputs
    ---------
    arr
        (2d np array) Large arry to downsample 
    sub_pixels_per_row
        (int) Number of pixels to bin by in the row direction
    sub_pixels_per_col
        (int) Number of pixels to bin by in the row direction


    Returns
    -------
    A 2d numpy array. If the input array has shape (nr,nc), the output
    shape is (nr/sub_pixels_per_row, nc/sub_pixels_per_col).

    Notes
    ------------
    * The input array shape must be a multiple of the sub pixels. For example,
    if the number of rows in the input array is 12, `sub_pixels_per_row` can
    be 1,2,3,4 or 6, but not 5.

    * Based on a Stack Overflow solution
    https://stackoverflow.com/questions/14916545/numpy-rebinning-a-2d-array

    """
    nr, nc = arr.shape
    num_col_out = nc // sub_pixels_per_col 
    num_row_out = nr // sub_pixels_per_row
    
    assert num_col_out * sub_pixels_per_col == nc 
    assert num_row_out * sub_pixels_per_row == nr
    temp = arr.reshape(num_row_out, sub_pixels_per_row, num_col_out,  sub_pixels_per_col)
    temp = temp.sum(axis=3).sum(axis=1)
    
    assert temp.shape == (num_row_out, num_col_out)
    return temp 






from tools import toFloat
import numpy as np
import const
import math


def abundanceFromMetalFraction(Z, dZ=None):
    """Compute abundunce, [X/H] from metal fraction, Z/(X+Y+Z)

    Theorists input, and report, the fraction of a solar composition
    that is composed of metals (elements heavier than Helium), as Z,
    with Z between 0 and 1.

    Observers report abundances relative to solar, or [Z/H] =
    log10(Z/H) - log10(Z/H)_solar

    This function converts from one to the other.
    """

    #Can't use toFloat here, because might confuse a bad value with a
    #legitimate one
    try:
        Z = float(Z)
        if dZ is not None:
            dZ = float(dZ)
    except ValueError:
        raise ValueError("Z (%s) and dZ (%s) must be floating point numbers" %
                (Z, dZ))

    Z0 = const.solarMetalFraction
    dZ0 = const.solarMetalFractionUnc

    feh = math.log10(Z) - math.log10(Z0)

    if dZ is None:
        return feh

    dfeh = (dZ/Z)**2 + (dZ0/Z0)**2
    dfeh *= 1/(math.log(10))**2
    dfeh = math.sqrt(dfeh)

    return feh, dfeh



def metalFractionFromAbundance(feh, dFeh=None):
    """Compute metal fraction, Z/(X+Y+Z), from abundunce, [Z/H]

    Theorists input, and report, the fraction of a solar composition
    that is composed of metals (elements heavier than Helium), as Z,
    with Z between 0 and 1.

    Observers report abundances relative to solar, or [Z/H] =
    log10(Z/H) - log10(Z/H)_solar

    This function converts from one to the other.
    """

    #Can't use toFloat here, because might confuse a bad value with a
    #legitimate one
    try:
        feh  = float(feh)
        if dFeh is not None:
            dFeh = float(dFeh)
    except ValueError:
        raise ValueError("feh (%s) and dFeh (%s) must be floating point numbers" %
                (feh, dFeh))

    Z0 = const.solarMetalFraction
    dZ0 = const.solarMetalFractionUnc

    val = 10**(feh)
    Z = Z0*val

    if dFeh is None:
        return Z

    a = val*dZ0
    b = Z0*val*math.log(10) * dFeh
    dZ = math.hypot(a,b)
    return Z, dZ



def estimateEllipsoidalAmplitude_AU(m1_solar, r1_solar, m2_solar, \
 period_days, incl_deg=90):
    """Wrapper for estimateEllipsoidalAmplitude_SI to allow
    user to pass values in astronomical units
    """

    mass1 = m1_solar * const.solarMass
    mass2 = m2_solar * const.solarMass
    radius1 = r1_solar * const.solarRadius

    period = period_days * 86400

    return estimateEllipsoidalAmplitude_SI(mass1, radius1, mass2, \
        period, incl_deg)


def estimateEllipsoidalAmplitude_SI(mass1, radius1, mass2, period, incl_deg=90):
    """Rough estimate of amplitude of ellipsoidal variation

    A companion to a star raises tides on its surface, changing the
    shape of the star to an ellipsoid and causing a
    periodic signal in the lightcurve. This function provides a crude
    estimate of the expected amplitude of this signal for a given system.

    The exact amplitude depends on the limb darkening coeff for a star,
    through the parameter beta. We just fix it to the value for Tres-2.

    Taken from Barclay et al (2012) submitted (on Tres-2).

    For calculation of semi-major axis see Carroll and Ostlie (1st edition)
    Eqn 2.35 (page 52).

    Inputs:
    (all params are floats, and in SI units (kg, m, seconds etc.)
    mass1
    radius1     Parameters of the primary on which tides are raised.
    mass2       Mass of the object raising the tides
    period      Orbital period of system
    incl_deg    Inclination of orbit to line of sight

    Returns:
    Expected fractional amplitude of ellipsoidal variation in
    """

    totalMass = mass1+mass2
    sma = astro.semiMajorAxis_SI(totalMass, period)

    aOverR = sma/radius1

    #For Tres-2, beta is 1.3 (see Barclay et al)
    #Because I'm only doing a quick estimate, hard code this number
    beta = 1.3

    sini = np.sin(np.deg2rad(incl_deg))

    Aell = mass2*beta*sini**2
    Aell /= mass1 * aOverR**3

    return Aell


def orbIncFromImpPar(impactPar, asemi_au, Rstar_solar, \
        impParUnc=None, asemiUnc=None, RstarUnc=None):
    """Compute orbital inclination give impact parameter

    Formula is b = a/R* cos(i)
    inverted gives i = acos(bR/a)

    Return
    if called without uncertainties returns orbital inclination
    in radians

    If called with uncertainties, returns inclination and uncertainty
    in radians.

    """

    #Rename variables and scale to SI
    b = toFloat(impactPar)
    R = toFloat(Rstar_solar)*const.solarRadius
    a = toFloat(asemi_au) * const.au

    val = b*R/a
    inc = np.arccos(val)

    #Now for uncertainties
    if impParUnc is None:
        return inc

    db = impParUnc
    dR = RstarUnc * const.solarRadius
    da = asemiUnc * const.au

    unc = (db/b)**2 + (dR/R)**2 + (da/a)**2
    unc *= (val**2)/(1-val**2)
    unc = np.sqrt(unc)

    return np.array([inc, unc])



def stellarMassFromLoggAndRadius(logg_cgs, radius_solar, loggUnc=None, radiusUnc=None):
    logg_SI = float(logg_cgs) -2   #Convert cgs to SI
    r_SI = float(radius_solar) * const.solarRadius
    mass_SI = (r_SI**2)/const.newton*(10**logg_SI)

    solarMass = mass_SI/const.solarMass
    if loggUnc is None:
        return solarMass

    dr_SI = float(radiusUnc) * const.solarRadius

    a = 2*mass_SI * dr_SI/r_SI
    b = mass_SI * math.log(10)*float(loggUnc)
    dsolarMass = math.hypot(a,b) / const.solarMass

    return solarMass, dsolarMass



def loggCgsFromMassRadius_solar(mass_solar, massUnc, radius_solar, radiusUnc):

    m_SI = mass_solar * const.solarMass
    mUnc = massUnc * const.solarMass

    r_SI = radius_solar * const.solarRadius
    rUnc = radiusUnc * const.solarRadius

    return loggCgsFromMassRadius(m_SI, mUnc, r_SI, rUnc)



def loggCgsFromMassRadius(mass, massUnc, radius, radiusUnc):
    #Gravity in m/s^2
    grav = const.newton*mass/radius**2

    #Convert to log cgs units (+2 converts from SI to cgs)
    logg = np.log10(grav) + 2

    #No need for a constant for uncertainty because we're dealing with logs
    #Note d/dx log10(y) = log10(e)/y
    unc = (massUnc/mass)**2 + (2*radiusUnc/radius)**2
    unc = np.log10(np.exp(1)) * np.sqrt(unc)


    return logg, unc


def radiusSolarFromMassSolarAndLoggCgs(mass_solar, logg_cgs, teffUnc=None, \
    loggUnc=None):

    if teffUnc is not None:
        raise ImplementationError("Uncertainties not available yet")

    mass_SI = mass_solar*const.solarMass
    r = radiusFromMassAndLogg(mass_SI, logg_cgs, teffUnc, loggUnc)

    return r/const.solarRadius


def radiusFromMassAndLogg(mass_SI, logg_cgs, teffUnc=None, \
    loggUnc=None):

    #100 cm/s^2 = 1 m/s^2 => logg(SI) = logg(CGS) -2
    g_SI = 10**(logg_cgs - 2)

    gm = const.newton*mass_SI
    radius = np.sqrt(gm/g_SI)

    if teffUnc is not None:
        raise ImplementationError("Uncertainties not available yet")

    return radius


def radiusFromTeffAndLuminosity_solar(teff_Kelvin, luminosity_solar,
    teffUnc = None, luminosityUnc = None):

    lum_SI = luminosity_solar * const.solarLuminosity
    if teffUnc is None:
        rad = radiusFromTeffAndLuminosity_SI(teff_Kelvin, lum_SI)
        return rad/const.solarRadius
    else:
        dLum_SI = luminosityUnc * const.solarLuminosity
        rad, drad = radiusFromTeffAndLuminosity_SI(teff_Kelvin, lum_SI)
        rad /= const.solarRadius
        drad /= const.solarRadius
        return rad, drad




def radiusFromTeffAndLuminosity_SI(teff_Kelvin, luminosity_SI, teffUnc=None, luminosityUnc=None):

    #Easier to use names
    teff = float(teff_Kelvin)
    lum = float(luminosity_SI)
    sigma = const.stefanBoltzmann

    radius = math.sqrt(lum/(4*math.pi*sigma*(teff**4)))

    if teffUnc is None:
        return radius

    #This code isn't tested.
    #varA = radius/(2*lum) * luminosityUnc
    #varB = -2*radius/teff * teffUnc
    #drad = math.hypot(varA, varB)

    #return radius, drad



def eqTeff(rStar_solar, teffStar_K, asemi_AU, \
    rStarUnc=None, teffUnc=None, asemiUnc=None, redistrib=1, albedo=0.3):
    """
    Compute the equilibrium temperature of a planet

    Default values taken from Batalha (2013)

    """

    radStar = float(rStar_solar) * const.solarRadius
    teffStar = float(teffStar_K)
    asemi = float(asemi_AU) * const.au

    C = redistrib*(1-albedo)
    C = C**.25
    Teq =  teffStar * (radStar/2./asemi)**.5 * C

    if rStarUnc is None:
        return Teq

    drad = float(rStarUnc) * const.solarRadius
    dteff = float(teffUnc)
    dasemi = float(asemiUnc) * const.au

    dTeq = (dteff/teffStar)**2 + (.5*drad/radStar)**2 + (-.5*dasemi/asemi)**2
    dTeq = np.sqrt(dTeq)
    dTeq *= Teq

    return Teq, dTeq



def radialVelocity_SI(m1_SI, m2_SI, period_sec, inclination_deg=90):
    """Max Radial velocity of object of mass m1 if it is orbitted by
    and object of mass m2 with the given period.

    Test:
    .2Mo around 1Mo with period of 20 days give 13892m/s
    Jupiter in 11.2 year orbit is about 10m/s
    """

    sini = np.sin(inclination_deg * np.pi/180)

    val = m2_SI**3
    val /= (m1_SI + m2_SI)**2

    val *= 2*np.pi*const.newton/period_sec

    exponent = 1/3.
    val = val**(exponent) * sini
    return val


def semiMajorAxis_AU(stellarMass_solar, period_days, sMassUnc=None, periodUnc=None):

    sMass = stellarMass_solar*const.solarMass
    period = period_days * 86400

    if sMassUnc is not None:
        sMassUnc *= const.solarMass

    if periodUnc is not None:
        periodUnc *= 86400

    val = semiMajorAxis_SI(stellarMass_solar, period_days, sMassUnc, periodUnc)

    try:
        for i in range(len(val)):
            val[i] = val[i]/const.au
    except TypeError:
        val /= const.au

    return val




def semiMajorAxis(stellarMass_solar, period_days, sMassUnc=None, periodUnc=None):
    """Deprecated. Used semiMajorAxis_AU or _SI instead"""
    #Convert to SI
    mass = float(stellarMass_solar)* const.solarMass
    period = float(period_days) * const.secondsPerDay

    if sMassUnc is not None:
        sMassInc = float(sMassUnc*const.solarMass)

    if periodUnc is not None:
        periodUnc = float(periodUnc) * const.secondsPerDay

    return semiMajorAxis_AU(mass, period, sMassUnc, periodUnc)



def semiMajorAxis_SI(stellarMass, period, sMassUnc=None, periodUnc=None):
    """Compute semi major axis

        Inputs:
        stellarMass     (float) Mass of central body in system (kg)
        period          (float) orbital period (seconds)
        sMassUnc        (float) Uncertainty in stellar mass (kg)
        periodUnc       (float) Uncertainty in period (kg)

        Returns
        Semi major axis in metres. If uncertainties are provided, an
        array of semi-major axis and it's uncertainty are returned.

        Note:
        Adapted from Eqn 2.35 of Carroll & Ostlie 1st Edition (page 52).

        If the mass of the 2nd body is a significant fraction of the
        primary, replace stellar mass with the sum of the masses
        in the input arguments for a more physical answer.
    """

    #Use easy to read names
    G = const.newton
    pi = math.pi
    mass = float(stellarMass)
    period = float(period)

    #Do the calculation
    val = G*mass*period**2
    val /= 4*pi**2
    asemi = val**(1/3.)

    if sMassUnc is None:
        return asemi

    dmass = float(sMassUnc) * const.solarMass
    dperiod = float(periodUnc) * const.secondsPerDay

    val = (dmass/3./mass)**2
    val += (2*dperiod/3./period)**2
    val = asemi*np.sqrt(val)

    return [asemi, val]



def stellarLumFromTeffAndRadius(teff_K, Rstar_solar, teffUnc=None, RstarUnc=None):
    """Compute stellar luminosity

    Formula is L = 4 pi sigma R**2 T**4

    Return
    if called without uncertainties returns luminosity
    in solar units

    If called with uncertainties, returns L and uncertainty
    in solar units

    """

    #Rename variables for convenience
    T = float(teff_K)
    R = float(Rstar_solar) * const.solarRadius
    sigma = const.stefanBoltzmann
    pi = np.pi

    lum = 4*pi*sigma*R**2*T**4
    lum /= const.solarLuminosity

    if teffUnc is None:
        return lum

    dT = float(teffUnc)
    dR = float(RstarUnc) * const.solarRadius

    val = (2*dR/R)**2 + (4*dT/T)**2
    val = np.sqrt(val)
    unc = val * lum

    return lum, val



def reducedMass(mass1, mass2):
    """Calculate the reduced mass of a 2 body system

    Inputs:
    mass(i)     Masses of the two objects.

    Return:
    Reduced mass in same units as mass(i)

    Notes:
    Units not important as long as m1 and m2 are in the same units

    The result in symettric in the arguments, i.e foo(m1, m2) == foo(m2, m1)
    """

    mu = (mass1*mass2)/(mass1+mass2)
    return mu



def wdMassLoggToRadius_m(mass_solar, logg_cgs):
    """Convert logg to a radius estimate"""
    mass_kg = mass_solar * const.solarMass
    logg_SI = logg_cgs - 2

    grav = 10**(logg_SI)
    radius_m = np.sqrt(const.newton * mass_kg/ grav)
    return radius_m





#A list of useful constants
#All are in the appropriate SI units

speedOfLight=2.99792458e8
newton=6.67259e-11
planck=6.6260755e-34
hBar=1.05457266e-34 # h/2pi
avogadro=6.0221367e23
boltzmann=1.380658e-23
stefanBoltzmann=5.67e-8 #W.m^-2.K^-4

electronCharge=1.6e-19
electronMass=9.1e-31
protonMass=1.67e-27
neutronMass=1.6749286e-27

planckMass=2.17671e-8
planckLength=1.61605e-35
planckTime=5.39056e-44

#Checked against \S 9.2 of Cambridge Handbook of Physical Formulae
#2010-12-06.
au = 1.495979e11
lightYear = 9.46073e15
parsec = 3.08420e+16
yearInSeconds = 31557600.0

#Values with the comment suffix M15 are taken from
#Mamajek et al (2015, arXiv:1510.07674)
solarMass = 1.988475e30          #M15
solarRadius = 6.957e8            #M15
solarConstant = 1361             #M15, W/m^2
solarLuminosity = 3.828e26       #M15
solarSurfaceTemperature = 5772.  #M15
solarSurfaceLogg = 4.437         #Not checked yet

#Some disagreement about the true value for these quantities.
#See solarconst in utils/mod/ms/ for a good discussion. These
#values from Antia & Basu (2006, ApJ 644 1292)
solarMetalFraction = 0.0172
solarMetalFractionUnc = 0.002

jupiterMass = 1.89818e27             #M15
jupiterRadius = 7.1492e7             #M15  Equatorial radius.

earthMass = 5.97236e24               #M15
earthPolarRadius = 6.3568e6          #M15
earthEquatorialRadius = 6.3781e6     #M15
earthRadius = earthEquatorialRadius  #Arbitary choice. Does it matter?

moonMass=7.3483e22
moonRadius=1.7374e6
moonDistance=3.84400e8

#Equatorial radii, polar is a little less
#Taken from the wikipedia page for each planet.
marsRadius =    3396*1e3
saturnRadius =  60268 * 1e3
neptuneRadius = 24764 * 1e3
uranusRadius =  25559 * 1e3




secondsPerDay=86400.
secondsPerYear=31557600.

#1Jy is 1e-26 W/m^2/Hz
jansky_SI = 1e-26


#Kepler constants
keplerFrameTime_s = 6.53877
keplerShortCadence_s = 58.8489
keplerLongCadence_s = 1765.4679

keplerShortCadence_days = keplerShortCadence_s / float(secondsPerDay)
keplerLongCadence_days = keplerLongCadence_s / float(secondsPerDay)



from ipdb import set_trace as idebug
from pdb import set_trace as debug
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

import matplotlib.colors as mcolor
import astropy.io.fits as pyfits
import scipy.ndimage as spimg
import matplotlib as mpl


"""The the filter tracings from the Excel (!) file provided by the
instrument makers

    http://ircamera.as.arizona.edu/MIRI/pces.htm
"""

def main():
    fn = "ImPCE_TN-00072-ATC-Iss2.csv"
    data = np.loadtxt(fn, skiprows=2, delimiter=',')
    data[:,0] *= 1e-6  #Convert to metres

    cols = getColNames(fn)

    for i in range(1, len(cols)):
        print(cols[i])
        trace = np.vstack([data[:,0], data[:,i]]).transpose()
        filename = "miri-%s.dat" %(cols[i])

        np.savetxt(filename, trace, fmt="%.6e")

def getColNames(fn):
    with open(fn) as fp:
        line = fp.readline()[:-1]

    cols = line.split(',')
    return cols

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['frmastro',
 'frmastro.filter_tracings.orig.jwst',
 'frmastro.filter_tracings.orig.uvot',
 'frmastro.psf']

package_data = \
{'': ['*'],
 'frmastro': ['filter_tracings/*',
              'filter_tracings/orig/*',
              'spec_mods/*',
              'spec_mods/bsl/*']}

install_requires = \
['astropy>=5.3.2,<6.0.0',
 'frmbase>=0.2.0,<0.3.0',
 'ipdb>=0.13.13,<0.14.0',
 'matplotlib>=3.7.2,<4.0.0',
 'numpy',
 'pandas',
 'pytest>=7.4.0,<8.0.0',
 'scipy']

setup_kwargs = {
    'name': 'frmastro',
    'version': '0.1.1',
    'description': "Fergal's astro related code in Python",
    'long_description': None,
    'author': 'Fergal',
    'author_email': 'fergal.mullally@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/python

import matplotlib.pyplot as mpl
import numpy as np
import pyfits

import const
def generateFilter(fitsFilename, outFilename, filterName=""):
    data = pyfits.getdata(fitsFilename)

    nrgLow = data['ENERG_LO']
    nrgHigh = data['ENERG_HI']
    transmission = data['MATRIX']

    nrg_kev = (nrgLow +nrgHigh)/2.
    nrg_ev = nrg_kev * 1000
    nrg_J = nrg_ev * const.electronCharge

    wavel = const.planck * const.speedOfLight / nrg_J

    #Transmission is given in "effective area". To get fractional transmission
    #divide by actual observing area. Taken from
    #http://heasarc.nasa.gov/docs/swift/about_swift/Sci_Fact_Sheet.pdf
    #at http://heasarc.nasa.gov/docs/swift/about_swift/
    totalArea = 659
    transmission /= totalArea

    #Test that each element of matrix is a float, and not a vector.
    #I don't know what to do with the vectors
    for t in transmission:
        assert( type(t) == np.float32)

    text = []
    text.append('#Swift UVOT %s. From makeUvot.py' %(filterName))
    text.append('#http://heasarc.nasa.gov/docs/swift/proposals/swift_responses.html')

    for w,t in zip(wavel, transmission):
        text.append( "%.3e %.3e" %(w, t))

    fp = open(outFilename, "w")
    fp.write( "\n".join(text))
    fp.close()
    print "Written to %s" %(outFilename)



def main():
    generateFilter('u.rsp', 'uvot_u.dat', filterName='U band')
    generateFilter('b.rsp', 'uvot_b.dat', filterName='B band')
    generateFilter('v.rsp', 'uvot_v.dat', filterName='V band')
    generateFilter('uvw1.rsp', 'uvot_w1.dat', filterName='W1 band')
    generateFilter('uvm2.rsp', 'uvot_m2.dat', filterName='M2 band')
    generateFilter('uvw2.rsp', 'uvot_w2.dat', filterName='W2 band')
    generateFilter('white.rsp', 'uvot_white.dat', filterName='white light')




"""
Stub of a module to interface with my model spectra libraries 

Notes:
See miri_wd_worlds/fergal/py/modelspec.py for some functions to deal with
other, not-yet-implemented models
"""

from frmbase.support import lmap 
import astropy.units as u 
from glob import glob 
import pandas as pd 
import numpy as np 
from astro import synth 
from astro import const 

import os 

class AbstractSpectrumCollection():
    def __init__(self, path=None, relpath=None, suffix=""):
        if not path:
            path = get_module_path()
            path = os.path.join(path , relpath)
        self.path = path 
        
        self.flist = glob(self.path + "*"+suffix)
        assert self.flist
        self.df = self.getModelList()
                    
    def getModelList(self) -> pd.DataFrame:
        """Returns a Dataframe of filenames, along with model parameters"""
        pass 
    
    
    def getModel(self) -> np.ndarray:
        pass 
    
    def interp(self, *pars):
        pass 
    


class  Atmo2020(AbstractSpectrumCollection):
    def __init__(self, path):
        AbstractSpectrumCollection.__init__(self, path, suffix=".txt")

    def getModelList(self):
        pars = lmap(self.getParsFromFilename, self.flist)
        df = pd.DataFrame()
        df['teff'] = lmap(lambda x: x[0], pars)
        df['logg'] = lmap(lambda x: x[1], pars)
        df['path'] = self.flist 
        return df
            
        return self.df

    def getParsFromFilename(self, fn):
        tokens = fn.split('_')
        teff = int(tokens[1][1:])
        logg = float(tokens[2][2:])
        return teff, logg 

    def getModel(self, teff, logg, radius_jup) ->np.ndarray:
        """
        TODO What is output format?
        
        Numpy array or pandas.
        microns or metres
        Jy or mJy
        """
        
        df = self.df 
        idx = df.teff == teff
        idx &= df.logg == logg 
        df = df[idx]
        assert len(df) == 1
        url = df.path.iloc[0]
        data = np.loadtxt(url, skiprows=2)
        data = self._convertTomJy(data, radius_jup)
        return data


    def _convertTomJy(self, atmo, radius_jup=1) -> np.ndarray:
        w_um = atmo[:,0]
        f_flam = atmo[:,1]

        w = w_um * u.micron
        f_lam = f_flam * u.Watt / u.m**2 / u.micron
        fjy = flam_to_jy(w, f_lam)

        #Flux from entire hemisphere at 10pc
        radius = radius_jup * const.jupiterRadius
        dist = 10 * const.parsec

        fjy *= (radius/dist)**2
        fmjy = fjy * 1e3

        out = np.zeros((len(atmo), 2))
        out[:,0] = w_um
        out[:,1] = fmjy        
        return out 

class  Bsl02(AbstractSpectrumCollection):
    """
    Exoplanet atmosphere models from 
    Burrows, Sudarsky & Luninie (2002)
    """
    
    def __init__(self, path=None):
        AbstractSpectrumCollection.__init__(self, path, "spec_mods/bsl/fort")

    def getModelList(self) -> pd.DataFrame:
        """Returns a Dataframe of filenames, along with model parameters"""
        
        pars = lmap(self.getParsFromFilename, self.flist)
        df = pd.DataFrame()
        df['mass'] = lmap(lambda x: x[0], pars)
        df['age'] = lmap(lambda x: x[1], pars)
        df['cloudy'] = lmap(lambda x: x[2], pars)
        df['path'] = self.flist 

        df['mass'] = df.mass.astype(int)
        df['age'] = df.age.astype(float)
        self.df = df 
            
        return self.df
    
    def getModel(self, mass, age, cloudy):
        """
        TODO What is output format?
        
        Numpy array or pandas.
        microns or metres
        Jy or mJy
        """
        
        df = self.df 
        idx = df.mass == mass
        idx &= df.age == age 
        idx &= df.cloudy == cloudy 
        # import ipdb; ipdb.set_trace()
        
        df = df[idx]
        assert len(df) == 1
        url = df[idx].path.iloc[0]
        data = np.loadtxt(url, skiprows=2)
        return data[:, [2,5]]
        


    def getParsFromFilename(self, f):
        tokens = f.split('_')
        mass = tokens[-2]
        age = tokens[-1][:3]
        cloudy = tokens[-1][3:]
        # import pdb; pdb.set_trace()
        return mass, age, cloudy
    
    
    
def get_module_path():
    path = os.path.split(__file__)[0]
    return path 



def flam_to_jy(w_u, f_flam):
    """Convert flux in wavelength units to Janskys

    This function operates on wavelengths and fluxes with
    astropy units attached. See loadAtmo2020 for an example of usage
    """
    eqv = u.spectral_density(w_u)

    #Flux in Jy per unit surface area
    fjy = f_flam.to(u.Jansky, eqv)
    return fjy
    